% ellipdeg.m - solves the degree equation in analog elliptic filter design
%
% Usage: k = ellipdeg(N,k1,tol);  
%        k = ellipdeg(N,k1,M);    (M = fixed number of Landen iterations)
%        k = ellipdeg(N,k1)       (uses tol=eps)
%
% N = analog filter order
% k1 = elliptic modulus for stopband band, that is, k1 = ep/ep1
% tol = tolerance, e.g., tol=1e-10, default is tol = eps
%
% M = uses a fixed number of Landen interations, typically, M = 4-5
%
% k = elliptic modulus for transition band, that is, k = WB/W1
%
% Notes: solves the degree equation N*K'/K = K1'/K1 for k in terms of N,k1
%
%        to solve for k1, given N,k, apply it to the complementary modulus k'
%        to return the complementary modulus k1'. that is,
%
%            kc = sqrt(1-k^2)
%            k1c = ellipdeg(N,kc,tol)
%            k1 = sqrt(1-k1c^2)
% 
%        or, compactly, k1 = sqrt(1-ellipdeg(N,sqrt(1-k^2))^2)
%
%        it uses the property that F_N(w) = 1/k1 at w = 1/k

% -------------------------------------------------------------------------
% Copyright (c) 2005 by Sophocles J. Orfanidis
% 
% Address: Sophocles J. Orfanidis                       
%          ECE Department, Rutgers University          
%          94 Brett Road, Piscataway, NJ 08854-8058, USA
%
% Email:   orfanidi@ece.rutgers.edu
% Date:    June 15, 2005
% 
% Reference: Sophocles J. Orfanidis, "High-Order Digital Parametric Equalizer 
%            Design," J. Audio Eng. Soc., vol.53, pp. 1026-1046, November 2005.
%
% Web Page: http://www.ece.rutgers.edu/~orfanidi/hpeq
% 
% tested with MATLAB R11.1 and R14
% -------------------------------------------------------------------------

function k = ellipdeg(N,k1,tol)

if nargin==0, help ellipdeg; return; end
if nargin==2, tol=eps; end

L = floor(N/2); 

i = 1:L;  ui = (2*i-1)/N;

kc = sqrt(1-k1^2);			% complement of k1 

kp = kc^N * (prod(sne(ui,kc)))^4;       % complement of k

k = sqrt(1-kp^2);                     









