% ellipk - complete elliptic integral of first kind
% 
% Usage: [K,Kprime] = ellipk(k,tol)
%        [K,Kprime] = ellipk(k)          (equivalent to tol=eps)
%        [K,Kprime] = ellipk(k,M)        (equivalent to using M Landen iterations)
%
% k   = elliptic modulus 
% tol = tolerance, e.g., tol = 10^(-10), default value is tol = eps = 2.22e-16 
% tol = M = fixed number of Landen iterations resulting in length(v) = M
%
% K      = quarter period K(k)
% Kprime = quarter period K'(k) = K(k'), k' = sqrt(1-k^2)
% 
%
% Notes: first it constructs the Landen vector of descending moduli, v = landen(k), 
%        and then it computes K = prod(1+v)) * pi/2
%
%        the nome is q = exp(-pi*K'/K) 
%
%        ELLIPK produces the same answer as the built-in function ELLIPKE, K = ellipke(k^2)
% 
%        it uses the function LANDEN

% -------------------------------------------------------------------------
% Copyright (c) 2005 by Sophocles J. Orfanidis
% 
% Address: Sophocles J. Orfanidis                       
%          ECE Department, Rutgers University          
%          94 Brett Road, Piscataway, NJ 08854-8058, USA
%
% Email:   orfanidi@ece.rutgers.edu
% Date:    June 15, 2005
% 
% Reference: Sophocles J. Orfanidis, "High-Order Digital Parametric Equalizer 
%            Design," J. Audio Eng. Soc., vol.53, pp. 1026-1046, November 2005.
%
% Web Page: http://www.ece.rutgers.edu/~orfanidi/hpeq
% 
% tested with MATLAB R11.1 and R14
% -------------------------------------------------------------------------

function [K,Kprime] = ellipk(k,tol)

if nargin==0, help ellipk; return; end
if nargin==1, tol=eps; end

if k==0,
   K = pi/2; Kprime = Inf;
elseif k==1,
   K = Inf; Kprime = pi/2;
else
   v = landen(k,tol);  
   K = prod(1+v) * pi/2;

   kprime = sqrt(1-k^2);
   vprime = landen(kprime,tol); 
   Kprime = prod(1+vprime) * pi/2;
end


